// testing high-resolution timer
// This program demonstrates the accuracy comparison between standard clock(),
// std::chrono::high_resolution_clock and Timer class.
// It measures the elapsed time of each step in the loop. High precision timer
// should measure at least 1 millisecond difference.
//
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2006-04-20
// UPDATED: 2024-04-18
///////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <cmath>
#include <ctime>
#include <chrono>
#include "Timer.h"
using namespace std;

int main(int argc, char *argv[])
{
    double start, stop, tick1, tick2;

    // first, use std clock() function to measure elapsed time ////////////////
    start = stop = tick1 = tick2 = (double)clock(); // start timer and remember initial ticks

    while((stop-start)/CLOCKS_PER_SEC < 1)    // loop for 1 sec
    {
        cout << (tick2 - tick1) / CLOCKS_PER_SEC * 1000 << " ms." << endl;

        // reset timer
        tick1 = tick2;
        tick2 = stop = (double)clock();
    }

    // debug
    cout << "CLOCKS_PER_SEC: " << CLOCKS_PER_SEC << endl;
    cout << "CLK_TCK: " << CLK_TCK << endl;
    cout << "clock()/CLOCKS_PER_SEC: " << clock()/CLOCKS_PER_SEC << endl;

    // pause until user input
    cout << "\n\nPress Enter key to run std::chrono...\n\n";
    char c;
    cin.get(c);


    // second, use high_resolution_clock to measure elapsed time //////////////
    chrono::high_resolution_clock::time_point point1 = chrono::high_resolution_clock::now();
    chrono::high_resolution_clock::time_point point2 = point1;
    auto startPoint = point1;
    auto stopPoint = point1;

    while(chrono::duration<double>(stopPoint - startPoint).count() < 1)    // loop for 1 sec
    {
        // delta duration per iteration
        cout << chrono::duration<double>(point2 - point1).count() * 1000 << " ms." << endl;

        // reset timer
        point1 = point2;
        point2 = stopPoint = chrono::high_resolution_clock::now();
    }

    // debug
    chrono::high_resolution_clock::period period;           // std::ratio<num, den>
    double resolution = (double)period.num / period.den;    // sec per tick
    cout << "Resolution (Seconds Per Tick): " << resolution << endl;
    // duration per tick
    //chrono::duration<double, std::nano> nanoPerTick = chrono::high_resolution_clock::duration(1);
    chrono::duration<double, std::nano> nanoPerTick = chrono::steady_clock::duration(1);
    //chrono::duration<double, std::micro> microPerTick = chrono::high_resolution_clock::duration(1);
    chrono::duration<double, std::micro> microPerTick = chrono::steady_clock::duration(1);
    //chrono::duration<double, std::milli> milliPerTick = chrono::high_resolution_clock::duration(1);
    chrono::duration<double, std::milli> milliPerTick = chrono::steady_clock::duration(1);
    cout << "Milli-seconds Per Tick: " <<  milliPerTick.count() << endl;
    cout << "Micro-seconds Per Tick: " <<  microPerTick.count() << endl;
    cout << "Nano-seconds Per Tick: " <<  nanoPerTick.count() << endl;

    // pause until user input
    cout << "\n\nPress Enter key to run Timer class...\n\n";
    cin.get(c);

    // third, use Timer::getElapsedTime() /////////////////////////////////////
    Timer t;

    // start timer
    t.start();
    tick1 = tick2 = t.getElapsedTimeInMilliSec();

    while(t.getElapsedTime() < 1)       // loop for 1 sec
    {
        cout << (tick2 - tick1) << " ms." << endl;

        tick1 = tick2;
        tick2 = t.getElapsedTimeInMilliSec();
    }

    return 0;
}
